@@ -22,6 +22,7 @@ __PACKAGE__->mk_accessors(
 use Array::Unique;
 use Carp qw(confess);
 use CPAN ();
+use CPAN::Meta;
 use Cwd qw( getcwd );
 use Debian::Control::FromCPAN;
 use Debian::Dependencies;
@@ -35,7 +36,7 @@ use Parse::DebianChangelog;
 use Text::Wrap qw(fill);
 use User::pwent;
 
-use constant debstdversion => '3.9.1';
+use constant debstdversion => '3.9.2';
 
 our %DEFAULTS = (
     start_dir => getcwd(),
@@ -98,6 +99,12 @@ sub debian_file {
     catfile( $self->main_file('debian'), $file );
 }
 
+sub build_pl {
+    my ($self) = @_;
+
+    return $self->main_file('Build.PL');
+}
+
 sub makefile_pl {
     my ($self) = @_;
 
@@ -158,38 +165,30 @@ sub fill_maintainer {
 sub process_meta {
     my ($self) = @_;
 
-    my $file = $self->main_file('META.yml');
-
-    # META.yml non-existent?
-    unless ( -f $file ) {
-        $self->meta({});
-        return;
-    }
+    $self->meta({});
 
     # Command line option nometa causes this function not to be run
     if( $self->cfg->nometa ) {
-        $self->meta({});
         return;
     }
 
-    my $yaml;
-
-    # YAML::LoadFile dies when it cannot properly parse a file - catch it in
-    # an eval, and if it dies, return -again- just an empty hashref.
-    eval { $yaml = YAML::LoadFile($file); };
-    if ($@) {
-        print "Error parsing $file - Ignoring it.\n";
-        print "Please notify module upstream maintainer.\n";
-        $yaml = {};
+    my $meta = $self->main_file('META.json');
+    if ( -e $meta ) {
+        print "Using META.json\n" if $self->cfg->verbose;
+    }
+    else {
+        $meta = $self->main_file('META.yml');
+        if ( -e $meta ) {
+            print "Using META.yml\n" if $self->cfg->verbose;
+        }
+        else {
+            print "WARNING: Neither META.json nor META.yml was found\n";
+            return;
+        }
     }
 
-    if (ref $yaml ne 'HASH') {
-        print "$file does not contain a hash - Ignoring it\n";
-        $yaml = {};
-    }
-
-    # Returns a simple hashref with all the keys/values defined in META.yml
-    $self->meta($yaml);
+    $meta = CPAN::Meta->load_file($meta);
+    $self->meta( $meta->as_struct );
 }
 
 sub set_package_name {
@@ -275,15 +274,18 @@ sub extract_basic {
     $self->debian_dir( $self->main_file('debian') );
 
     find(
-        sub {
-            return if $File::Find::name =~ $self->cfg->exclude;
+        {   no_chdir => 1,
+            wanted   => sub {
+                return if $File::Find::name =~ $self->cfg->exclude;
 
-            if (/\.(pm|pod)$/) {
-                $self->extract_desc($_)
-                    unless $bin->short_description and $bin->long_description;
-                $self->extract_basic_copyright($_)
-                    unless $self->author and $self->copyright;
-            }
+                if (/\.(pm|pod)$/) {
+                    $self->extract_desc($_)
+                        unless $bin->short_description
+                            and $bin->long_description;
+                    $self->extract_basic_copyright($_)
+                        unless $self->author and $self->copyright;
+                }
+            },
         },
         $self->main_dir
     );
@@ -303,7 +305,31 @@ sub extract_name_ver {
         }
     }
     else {
-        $self->extract_name_ver_from_makefile( $self->makefile_pl );
+        if ( -e $self->build_pl ) {
+            print "Extracting name and version from Build.PL\n";
+            $self->extract_name_ver_from_build( $self->build_pl );
+        }
+        elsif ( -e $self->makefile_pl ) {
+            print "Extracting name and version from Makefile.PL\n";
+            $self->extract_name_ver_from_makefile( $self->makefile_pl );
+        }
+        else {
+            if ( $self->cfg->cpan ) {
+                my $name = $self->cfg->cpan;
+                $name =~ s/::/-/g;
+                $self->perlname($name);
+            }
+            else {
+                die "Unable to determine dist name, no Build.PL, no Makefile.PL\nPlease use --cpan.\n";
+            }
+
+            if ( $self->cfg->version ) {
+                $self->version( $self->cfg->version );
+            }
+            else {
+                die "Unable to determine dist version, no Build.PL, no Makefile.PL\nPlease use --version.\n";
+            }
+        }
         $name = $self->perlname;
         $ver  = $self->version;
     }
@@ -328,6 +354,135 @@ sub extract_name_ver {
     $self->set_package_name;
 }
 
+sub extract_name_ver_from_build {
+    my ( $self, $build ) = @_;
+    my ( $file, $name, $ver, $vfrom, $dir );
+
+    {
+        local $/ = undef;
+        my $fh = $self->_file_r($build);
+        $file = $fh->getline;
+    }
+
+    # Replace q[quotes] by "quotes"
+    $file =~ s/q\[(.+)]/'$1'/g;
+
+    # Get the name
+    if ($file =~ /([\'\"]?)
+	    dist_name\1\s*
+	    (=>|,)
+	    \s*
+	    ([\'\"]?)
+	    (\S+)\3/xs
+        )
+    {
+        $name = $4;
+    }
+    elsif (
+        $file =~ /([\'\"]?)
+		 module_name\1\s*
+		 (=>|,)
+		 \s*
+		 ([\'\"]?)
+		 (\S+)\3/xs
+        )
+    {
+        $name = $4;
+        $name =~ s/::/-/g;
+
+        # just in case we need it later
+        $vfrom = $name;
+        $vfrom =~ s/-/::/g;
+        $vfrom =~s{::}{/}g;
+        $vfrom = "lib/$vfrom.pm";
+    }
+    $name =~ s/,.*$//;
+
+    # band aid: need to find a solution also for build in directories
+    # warn "name is $name (cpan name: $self->cfg->cpan)\n";
+    $name = $self->cfg->cpan     if ( $name eq '__PACKAGE__' && $self->cfg->cpan );
+    $name = $self->cfg->cpanplus if ( $name eq '__PACKAGE__' && $self->cfg->cpanplus );
+
+    # Get the version
+    if ( defined $self->cfg->version ) {
+
+        # Explicitly specified
+        $ver = $self->cfg->version;
+
+    }
+    elsif ( $file =~ /([\'\"]?)\sdist_version\1\s*(=>|,)\s*([\'\"]?)(\S+)\3/s ) {
+        $ver = $4;
+
+        # Where is the version taken from?
+        $vfrom = $4
+            if $file
+                =~ /([\'\"]?)dist_version_from\1\s*(=>|,)\s*([\'\"]?)(\S+)\3/s;
+
+    }
+    elsif ( $file =~ /([\'\"]?)dist_version_from\1\s*(=>|,)\s*([\'\"]?)(\S+)\3/s )
+    {
+        $vfrom = $4;
+
+    }
+
+    $dir = dirname($build) || './';
+
+    for ( ( $name, $ver ) ) {
+        next unless defined;
+        next unless /^\$/;
+
+        # decode simple vars
+        s/(\$\w+).*/$1/;
+        if ( $file =~ /\Q$_\E\s*=\s*([\'\"]?)(\S+)\1\s*;/ ) {
+            $_ = $2;
+        }
+    }
+
+    unless ( defined $ver ) {
+        local $/ = "\n";
+
+        # apply the method used by makemaker
+        if (    defined $dir
+            and defined $vfrom
+            and -f "$dir/$vfrom"
+            and -r "$dir/$vfrom" )
+        {
+            my $fh = $self->_file_r("$dir/$vfrom");
+            while ( my $lin = $fh->getline ) {
+                if ( $lin =~ /([\$*])(([\w\:\']*)\bVERSION)\b.*\=/ ) {
+                    no strict;
+
+                    #warn "ver: $lin";
+                    $ver = ( eval $lin )[0];
+                    last;
+                }
+            }
+            $fh->close;
+        }
+        else {
+            if ( $self->mod_cpan_version ) {
+                $ver = $self->mod_cpan_version;
+                warn "Cannot use internal module data to gather the "
+                    . "version; using cpan_version\n";
+            }
+            else {
+                die "Cannot use internal module data to gather the "
+                    . "version; use --cpan or --version\n";
+            }
+        }
+    }
+
+    $self->perlname($name);
+    $self->version($ver);
+
+    $self->set_package_name;
+
+    if ( defined($vfrom) ) {
+        $self->extract_desc("$dir/$vfrom");
+        $self->extract_basic_copyright("$dir/$vfrom");
+    }
+}
+
 sub extract_name_ver_from_makefile {
     my ( $self, $makefile ) = @_;
     my ( $file, $name, $ver, $vfrom, $dir );
@@ -1127,7 +1282,7 @@ sub module_build {
 Adds the list of dependencies to I<$dependencies> and shows I<$reason> if in
 verbose mode.
 
-Used to both bump a dependency ant tell the user why.
+Used to both bump a dependency and tell the user why.
 
 I<$dependencies> is an instance of L<Debian::Dependencies> class, and
 I<@dependencies> is a list of L<Debian::Dependency> instances or strings.
@@ -1161,10 +1316,6 @@ sub configure_cpan {
     unshift( @{ $CPAN::Config->{'urllist'} }, $self->cfg->cpan_mirror )
         if $self->cfg->cpan_mirror;
 
-    $CPAN::Config->{'build_dir'}         = $ENV{'HOME'} . "/.cpan/build";
-    $CPAN::Config->{'cpan_home'}         = $ENV{'HOME'} . "/.cpan/";
-    $CPAN::Config->{'histfile'}          = $ENV{'HOME'} . "/.cpan/history";
-    $CPAN::Config->{'keep_source_where'} = $ENV{'HOME'} . "/.cpan/source";
     $CPAN::Config->{'tar_verbosity'}     = $self->cfg->verbose ? 'v' : '';
     $CPAN::Config->{'load_module_verbosity'}
         = $self->cfg->verbose ? 'verbose' : 'silent';
@@ -1443,13 +1594,15 @@ sub _file_w {
 
 =item Copyright (C) 2006 Frank Lichtenheld <djpig@debian.org>
 
-=item Copyright (C) 2007-2010 Gregor Herrmann <gregoa@debian.org>
+=item Copyright (C) 2007-2011 Gregor Herrmann <gregoa@debian.org>
 
 =item Copyright (C) 2007-2010 Damyan Ivanov <dmn@debian.org>
 
 =item Copyright (C) 2008, Roberto C. Sanchez <roberto@connexer.com>
 
-=item Copyright (C) 2009-2010, Salvatore Bonaccorso <carnil@debian.org>
+=item Copyright (C) 2009-2011, Salvatore Bonaccorso <carnil@debian.org>
+
+=item Copyright (C) 2011, Nicholas Bamber <nicholas@periapt.co.uk>
 
 =back
 
